@@ -12,6 +12,9 @@ from reportlab.pdfbase.ttfonts import TTFont
 from reportlab.pdfbase import pdfmetrics
 import SigProfilerExtractor as cosmic
 from PyPDF2 import PdfFileWriter, PdfFileReader, PdfFileMerger
+# imports for saving plots to memory
+import io
+from PIL import Image
 # imports for dashed line
 from reportlab.lib.colors import black
 paths = cosmic.__path__[0]
@@ -210,58 +213,56 @@ def draw_bracket(num_bases, c_draw):
 #	pdf_to_edit 	- (String)	The path to the uncropped decomposition plot
 #	output_name 	- (String)	The name to save the cropped output as
 #	num_bases		- (Integer)	The number of signatures the sample is composed of
-def crop_margins(pdf_to_edit, output_name, num_bases):
-	pdf_file = PdfFileReader(open(pdf_to_edit, "rb"))
+def crop_margins(pdf_to_edit, num_bases):
+	pdf_to_edit.seek(0)
+	pdf_file = PdfFileReader(pdf_to_edit, "rb")
 	page = pdf_file.getPage(0)
 	writer = PdfFileWriter()
+	output_plot_buff = io.BytesIO()
+	
 	if (num_bases == 1):
 		page.mediaBox.lowerRight = (792,150)
 		page.mediaBox.lowerLeft = (0,150)
 		page.mediaBox.upperRight = (792,402)
 		page.mediaBox.upperLeft = (0,402)
 		writer.addPage(page)
-		with open(output_name, "wb") as out_f:
-			writer.write(out_f)
+		writer.write(output_plot_buff)
 	elif (num_bases == 2):
 		page.mediaBox.lowerRight = (792,150)
 		page.mediaBox.lowerLeft = (0,150)
 		page.mediaBox.upperRight = (792,422)
 		page.mediaBox.upperLeft = (0,422)
 		writer.addPage(page)
-		with open(output_name, "wb") as out_f:
-			writer.write(out_f)
+		writer.write(output_plot_buff)
 	elif (num_bases == 3):
 		page.mediaBox.lowerRight = (792,150)
 		page.mediaBox.lowerLeft = (0,150)
 		page.mediaBox.upperRight = (792,462)
 		page.mediaBox.upperLeft = (0,462)
 		writer.addPage(page)
-		with open(output_name, "wb") as out_f:
-			writer.write(out_f)
+		writer.write(output_plot_buff)
 	elif (num_bases == 4):
 		page.mediaBox.lowerRight = (792,112)
 		page.mediaBox.lowerLeft = (0,112)
 		page.mediaBox.upperRight = (792,498)
 		page.mediaBox.upperLeft = (0,498)
 		writer.addPage(page)
-		with open(output_name, "wb") as out_f:
-			writer.write(out_f)
+		writer.write(output_plot_buff)
 	elif (num_bases == 5):
 		page.mediaBox.lowerRight = (792,75)
 		page.mediaBox.lowerLeft = (0,75)
 		page.mediaBox.upperRight = (792,537)
 		page.mediaBox.upperLeft = (0,537)
 		writer.addPage(page)
-		with open(output_name, "wb") as out_f:
-			writer.write(out_f)
+		writer.write(output_plot_buff)
 	elif (num_bases > 5):
 		page.mediaBox.lowerRight = (792,55)
 		page.mediaBox.lowerLeft = (0,55)
 		page.mediaBox.upperRight = (792,537)
 		page.mediaBox.upperLeft = (0,537)
 		writer.addPage(page)
-		with open(output_name, "wb") as out_f:
-			writer.write(out_f)
+		writer.write(output_plot_buff)
+	return output_plot_buff
 
 # Parameters:
 #   de_novo_name 				(String) 			The name of the denovo signature.
@@ -327,9 +328,9 @@ def gen_plot(de_novo_name, bases, output_path, project, c, reconstruction, \
 def gen_decomposition(denovo_name, basis_names, weights, output_path, project, \
 	denovo_plots_dict, basis_plots_dict, reconstruction_plot_dict, reconstruction, \
 	statistics,  sig_version=None, custom_text=None):
-	output_plot_to_crop = output_path+"/"+denovo_name+"_decomposition_"+project+"_to_crop.pdf"
-	output_plot_cropped = output_path+"/"+denovo_name+"_decomposition_"+project+".pdf"
-	c = canvas.Canvas(output_plot_to_crop, pagesize=letter)
+	
+	buff = io.BytesIO()
+	c = canvas.Canvas(buff, pagesize=letter)
 	c.setPageSize(landscape(letter))
 	c.setFont("Arial-Bold", 7.19)
 
@@ -337,7 +338,6 @@ def gen_decomposition(denovo_name, basis_names, weights, output_path, project, \
 	for i in range(0,len(basis_names)):
 		basis_plots.append([basis_names[i], weights[i]])
 
-
 	# create for loop to iterate through list, then change second value in list of lists
 	# Otherwise sorts strings and then 5.14% > 48.54%
 	for j in range(0, len(basis_names)):
@@ -349,8 +349,8 @@ def gen_decomposition(denovo_name, basis_names, weights, output_path, project, \
 		reconstruction_plot_dict)
 
 	c.save()
-
-	crop_margins(output_plot_to_crop, output_plot_cropped, len(basis_names))
-
-	if os.path.exists(output_plot_to_crop):
-			os.remove(output_plot_to_crop)
+	
+	# Take the plot and crop the margins
+	byte_plot = crop_margins(buff, len(basis_names))
+	buff.close()
+	return byte_plot
