@@ -41,6 +41,8 @@ import multiprocessing
 from SigProfilerExtractor import subroutines as sub
 import SigProfilerMatrixGenerator
 from SigProfilerMatrixGenerator.scripts import SigProfilerMatrixGeneratorFunc as datadump   
+from SigProfilerMatrixGenerator.scripts import SVMatrixGenerator as sv
+from SigProfilerMatrixGenerator.scripts import CNVMatrixGenerator as scna
 import multiprocessing as mp
 import SigProfilerExtractor as cosmic
 # from SigProfilerExtractor import single_sample as ss
@@ -143,6 +145,7 @@ def record_parameters(sysdata, execution_parameters, start_time):
     sysdata.write("\tstability: {}\n".format(execution_parameters["stability"]))
     sysdata.write("\tmin_stability: {}\n".format(execution_parameters["min_stability"]))
     sysdata.write("\tcombined_stability: {}\n".format(execution_parameters["combined_stability"]))
+    sysdata.write("\tallow_stability_drop: {}\n".format(execution_parameters["allow_stability_drop"]))
     
     sysdata.write("COSMIC MATCH\n")
     sysdata.write("\topportunity_genome: {}\n".format(execution_parameters["opportunity_genome"]))
@@ -150,7 +153,6 @@ def record_parameters(sysdata, execution_parameters, start_time):
     sysdata.write("\tnnls_add_penalty: {}\n".format(execution_parameters["nnls_add_penalty"]))
     sysdata.write("\tnnls_remove_penalty: {}\n".format(execution_parameters["nnls_remove_penalty"]))
     sysdata.write("\tinitial_remove_penalty: {}\n".format(execution_parameters["initial_remove_penalty"]))
-    sysdata.write("\tde_novo_fit_penalty: {}\n".format(execution_parameters["de_novo_fit_penalty"])) 
     sysdata.write("\trefit_denovo_signatures: {}\n".format(execution_parameters["refit_denovo_signatures"]))
     sysdata.write("\tcollapse_to_SBS96: {}\n".format(execution_parameters["collapse_to_SBS96"]))
     
@@ -182,7 +184,6 @@ def sigProfilerExtractor(input_type,
                          nmf_tolerance= 1e-15, 
                          nnls_add_penalty=0.05, 
                          nnls_remove_penalty=0.01,
-                         de_novo_fit_penalty=0.02,
                          initial_remove_penalty=0.05,
                          refit_denovo_signatures=True,
                          collapse_to_SBS96=True,
@@ -192,6 +193,7 @@ def sigProfilerExtractor(input_type,
                          stability=0.8, 
                          min_stability=0.2, 
                          combined_stability=1.0,
+                         allow_stability_drop=False,
                          get_all_signature_matrices= False): 
     """
     Extracts mutational signatures from an array of samples.
@@ -270,16 +272,16 @@ def sigProfilerExtractor(input_type,
 
     combined_stability: A float. Default is 1.0. The cutoff thresh-hold of the combined stability (sum of average and minimum stability). Solutions with combined stabilities below this thresh-hold will not be considered.            
     
+    allow_stability_drop: Boolean, optional. Default is False. Defines if solutions with a drop in stability with respect to the highest stable number of signatures will be considered.
+
     
     DECOMPOSITION:-
     
-    de_novo_fit_penalty: Float, optional. Takes any positive float. Default is 0.02. Defines the weak (remove) thresh-hold cutoff to be assigned denovo signatures to a sample. 
+    nnls_add_penalty: Float, optional. Takes any positive float. Default is 0.05. Defines the strong (add) thresh-hold cutoff to be assigned signatures to a sample. 
     
-    nnls_add_penalty: Float, optional. Takes any positive float. Default is 0.05. Defines the strong (add) thresh-hold cutoff to be assigned COSMIC signatures to a sample. 
-    
-    nnls_remove_penalty: Float, optional. Takes any positive float. Default is 0.01. Defines the weak (remove) thresh-hold cutoff to be assigned COSMIC signatures to a sample.
+    nnls_remove_penalty: Float, optional. Takes any positive float. Default is 0.01. Defines the weak (remove) thresh-hold cutoff to be assigned signatures to a sample.
      
-    initial_remove_penalty: Float, optional. Takes any positive float. Default is 0.05. Defines the initial weak (remove) thresh-hold cutoff to be COSMIC assigned signatures to a sample.
+    initial_remove_penalty: Float, optional. Takes any positive float. Default is 0.05. Defines the initial weak (remove) thresh-hold cutoff to be assigned COSMIC signatures to a sample.
     
     refit_denovo_signatures: Boolean, optional. Default is False. If True, then refit the denovo signatures with nnls.
     
@@ -328,7 +330,7 @@ def sigProfilerExtractor(input_type,
     start_time = datetime.datetime.now()
     
     #set the output variable
-    out_put = output; 
+    out_put = output 
     
     if gpu == True:
         import torch
@@ -391,7 +393,6 @@ def sigProfilerExtractor(input_type,
                         "nnls_add_penalty":nnls_add_penalty,
                         "nnls_remove_penalty":nnls_remove_penalty,
                         "initial_remove_penalty":initial_remove_penalty,
-                        "de_novo_fit_penalty":de_novo_fit_penalty,
                         "refit_denovo_signatures":refit_denovo_signatures,
                         "collapse_to_SBS96":collapse_to_SBS96,
                         "dist":clustering_distance,
@@ -400,6 +401,7 @@ def sigProfilerExtractor(input_type,
                         "stability":stability, 
                         "min_stability":min_stability, 
                         "combined_stability":combined_stability,
+                        "allow_stability_drop":allow_stability_drop,
                         "get_all_signature_matrices":get_all_signature_matrices}
                         
     ################################ take the inputs from the general optional arguments ####################################
@@ -433,7 +435,7 @@ def sigProfilerExtractor(input_type,
             seed  = np.array(seeds["Seed"])
 
         except:
-            "Please set valid seeds"
+            raise ValueError("Please set valid seeds")
     
     if input_type=="text" or input_type =="table" or input_type=="matrix":
         
@@ -456,17 +458,16 @@ def sigProfilerExtractor(input_type,
             else:
                 orderlist1=list(feature_map[0])
                 orderlist2=list(feature_order[0])
-                #sort the mutation types first step
-                data["Mutation Types"]= pd.Categorical(data["Mutation Types"], orderlist1)
-                data = data.sort_values("Mutation Types")
+                #sort the MutationType first step
+                data["MutationType"]= pd.Categorical(data["MutationType"], orderlist1)
+                data = data.sort_values("MutationType")
                 data = data.reset_index()
                 data = data.drop(columns='index')
-                #sort the mutation types second step
-                data["Mutation Types"]=feature_map[1]
-                data["Mutation Types"]= pd.Categorical(data["Mutation Types"], orderlist2)
-                data = data.sort_values("Mutation Types")
+                #sort the MutationType second step
+                data["MutationType"]=feature_map[1]
+                data["MutationType"]= pd.Categorical(data["MutationType"], orderlist2)
+                data = data.sort_values("MutationType")
                 
-        
         data = data.dropna(axis=1, inplace=False)
         data = data.loc[:, (data != 0).any(axis=0)]
         genomes = data.iloc[:,1:]
@@ -536,9 +537,9 @@ def sigProfilerExtractor(input_type,
 
     elif input_type=="vcf":
         ################################# For vcf input files #######################################################
-        title = project # set the title for plotting 
+        title = project # set the title for plotting
         data = datadump.SigProfilerMatrixGeneratorFunc(project_name, refgen, project, exome=exome,  bed_file=None, chrom_based=False, plot=False, gs=False)
-        # Selecting the mutation types    
+        # Selecting the MutationType
         if mtype == ["default"]:
             mtypes = ["SBS96", "DBS78", "ID83"] 
         elif mtype == "default":
@@ -549,7 +550,33 @@ def sigProfilerExtractor(input_type,
             mtype = mtype.replace(" ", "")
             mtypes = mtype.split(",")
         #set the genome_build
-        genome_build=refgen 
+        genome_build=refgen
+    elif input_type.lower()=="bedpe":
+        ##################### For SV's bedpe input files #####################
+        # create a directory to write the output matrices to
+        mtypes = ["SV32"]
+        sv_outputs = os.path.join(os.path.split(input_data)[0], "SV_Matrices")
+
+        # SV input processing, execution parameters
+        genomes = sv.generateSVMatrix(project, project_name, sv_outputs)
+        index = genomes.index.values
+        colnames = genomes.columns
+        allcolnames = colnames.copy()
+        allgenomes = genomes.copy()
+    elif input_type.split(":")[0].lower()=="seg": # seg
+        ################# For CNV's segmentation input files #################
+        title = project
+        mtypes = ["CNV48"]
+        cnv_file_type = input_type.split(":")[1].upper()
+        # cnv_outputs = os.path.join(os.path.split(input_data)[0], "CNV_Matrices")
+
+        # SV input processing, execution parameters
+        #project needs to be something NOT a file
+        genomes = scna.generateCNVMatrix(cnv_file_type, input_data, cnv_file_type, output)
+        index = genomes.index.values
+        colnames = genomes.columns
+        allcolnames = colnames.copy()
+        allgenomes = genomes.copy()
     else:
         raise ValueError("Please provide a correct input_type. Check help for more details")
     
@@ -560,7 +587,6 @@ def sigProfilerExtractor(input_type,
     ###########################################################################################################################################################################################                  
     
     for m in mtypes:
-        mutation_context = m
         # we need to rename the m because users input could be SBS96, SBS1536, DBS78, ID83 etc
         if m.startswith("SBS"):
             m = m[3:] #removing "SBS"
@@ -617,7 +643,7 @@ def sigProfilerExtractor(input_type,
                 print("Sample is not a nozero matrix for the mutation context "+ m)
                 sysdata.close()
                 continue
-                
+
             genomes = genomes.loc[:, (genomes != 0).any(axis=0)]
             allgenomes = genomes.copy()  # save the allgenomes for the final results 
             index = genomes.index.values
@@ -770,7 +796,7 @@ def sigProfilerExtractor(input_type,
 
         ########################################## Plot Stabiltity vs Reconstruction Error #############################        
         # Print the Stabiltity vs Reconstruction Error as get the solution as well
-        solution, all_stats = sub.stabVsRError(layer_directory+"/All_solutions_stat.csv", layer_directory, title, all_similirities_list, mtype=mutation_type, stability=stability, min_stability=min_stability, combined_stability=combined_stability, sequence=sequence)
+        solution, all_stats = sub.stabVsRError(layer_directory+"/All_solutions_stat.csv", layer_directory, title, all_similirities_list, mtype=mutation_type, stability=stability, min_stability=min_stability, combined_stability=combined_stability, sequence=sequence, allow_stability_drop=allow_stability_drop)
         all_stats.insert(1, 'Stability (Avg Silhouette)', minimum_stabilities) #!!!!!!!!!!!!!!!!1 here minimum stability is avg stability
         all_stats=all_stats.set_index(["Signatures"])
         all_stats.to_csv(layer_directory+"/All_solutions_stat.csv", sep = ",")
@@ -779,7 +805,7 @@ def sigProfilerExtractor(input_type,
         layer_genome = pd.DataFrame(genomes)
         layer_genome = layer_genome.set_index(index)
         layer_genome.columns = colnames
-        layer_genome = layer_genome.rename_axis("Mutation Types", axis="columns")
+        layer_genome = layer_genome.rename_axis("MutationType", axis="columns")
 
         # record the samples
         layer_genome.to_csv(output+"/Samples.txt", sep = "\t", index_label=[layer_genome.columns.name])
@@ -796,7 +822,7 @@ def sigProfilerExtractor(input_type,
 
         signature_stabilities = sub.signature_plotting_text(signature_stabilities, "Stability", "float")
         signature_total_mutations = sub.signature_plotting_text(signature_total_mutations, "Total Mutations", "integer")
-        listOfSignatures = sub.make_letter_ids(idlenth = processAvg.shape[1], mtype=mutation_context)
+        listOfSignatures = sub.make_letter_ids(idlenth = processAvg.shape[1], mtype=mutation_type)
 
         layer_directory1 = output+"/Suggested_Solution/"+mutation_type+"_De-Novo_Solution"
         layer_directory2 = output+"/Suggested_Solution/COSMIC_"+mutation_type+"_Decomposed_Solution"
